<?php
if ( ! defined( 'WPINC' ) ) die ;
?>
<h3 class="litespeed-title"><?php echo __( 'Network Media Settings', 'litespeed-cache' ) ; ?></h3>

<table><tbody>

	<?php require LSCWP_DIR . 'admin/tpl/setting/settings_inc.media_webp.php' ; ?>

</tbody></table>
